#'Function to create single system/reference scorecards
#' 
#'@description Scorecards function to create scorecard table for one system and 
#'  one reference combination.
#'@param aggregated_metrics is an array of spatially aggregated metrics loaded 
#'  in the function SC_spatial_aggregation.

#'@example
# scorecard_table <- SC_create_table(aggregated_metrics,
#                                    system.name = list('ECMWF System 5','DWD System 21'),
#                                    reference.name = 'ERA5',
#                                    start.year = 1993,
#                                    end.year = 2016,
#                                    var.name = 'tas', 
#                                    var.unit = 'K',
#                                    table.column.header
#                                    )
 
  
SC_create_table <- function(input_data,
                            system,
                            reference,
                            var,
                            start.year,
                            end.year,
                            start.months,
                            forecast.months,
                            region.names,
                            metrics,
                            fileout){
  
  input_data <- aggregated_metrics
  system <- c('ecmwfs5', 'dwds21')  
  reference <-'era5'
  var <- 'tas'
  start.year <- 1993
  end.year <- 2016
  start.months <- sprintf("%02d", 1:12)
  forecast.months <- 1:6
  region.names <- c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
  metrics <- c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr')
  fileout <- '/esarchive/scratch/nmilders/scorecards_data/saved_results/testing'
  
  setwd("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/")

    ## Checks to apply:
    # first dimension in aggregated_metrics is system and second dimension is reference
    # to allow 1 region - if region = 1 --> only scorecards 1 & 3 need to be plotted
  

    # start.months <- attributes(data)$start.months
    # regions.names <- names(attributes(data)$regions)
    # forecast.months <- attributes(data)$forecast.months  
    # reference <- attributes(data)$reference
    # system <- attributes(data)$system[1] ## selecting only first system for testing
      
    # Transform data for scorecards by forecast month (types 2 & 4)
    transformed_data <- SC_transform_data(data = input_data,
                                          sdate_dim = 'sdate',
                                          ftime_dim = 'time')

    ## Loop over system and reference for each scorecard plot
    for (sys in 1:dim(input_data)['system']){
      for (ref in 1:dim(input_data)['reference']) {
        
        #### Scorecard_type 1 ####
        ## (no transformation or reorder)
        data_sc_1 <- input_data[sys, ref,,,,]
        SC_plot_simple_by_region(data = data_sc_1,
                                 system = system[sys],
                                 reference = reference[ref],
                                 var = var,
                                 start.year = start.year,
                                 end.year = end.year,
                                 start.months = start.months,
                                 forecast.months = forecast.months,
                                 region.names = region.names,
                                 metrics = metrics,
                                 scorecard_type = 1,
                                 fileout =  fileout)

        
        #### Scorecard_type 2 ####
        ## (reorder only)
        ## Scorecard type 2 is same as type 1 for only one region, therefore is
        ## only plotted if more that one region is requested
        if (dim(aggregated_metrics)['region'] > 1) {
          new_order <- c(1,4,3,2) ## TO DO: reorder by dimension name not number
          data_sc_2 <- Reorder(aggregated_metrics[sys, ref,,,,], new_order)
          SC_plot_simple_by_forecast_month(data = data_sc_2,
                                           system = system[sys],
                                           reference = reference[ref],
                                           var = var,
                                           start.year = start.year,
                                           end.year = end.year,
                                           start.months = start.months,
                                           forecast.months = forecast.months,
                                           region.names = region.names,
                                           metrics = metrics,
                                           scorecard_type = 2,
                                           fileout =  fileout)
        } ## close if
        
        #### Scorecard_type 3 ####
        ## (transformation only)
        data_sc_3 <- transformed_data[sys, ref,,,,]
        SC_plot_simple_by_region(data = data_sc_3,
                                 system = system[sys],
                                 reference = reference[ref],
                                 var = var,
                                 start.year = start.year,
                                 end.year = end.year,
                                 start.months = start.months,
                                 forecast.months = forecast.months,
                                 region.names = region.names,
                                 metrics = metrics,
                                 scorecard_type = 3,
                                 fileout =  fileout)


        #### Scorecard_type 4 ####
        ## (transformation and reorder)
        ## Scorecard type 4 is same as type 3 for only one region, therefore is
        ## only plotted if more that one region is requested
        if(dim(aggregated_metrics)['region'] > 1) {
          new_order <- c(1,4,3,2) ## TO DO: reorder by dimension name not number
          data_sc_4 <- Reorder(transformed_metrics[sys, ref,,,,], new_order)
          SC_plot_simple_by_forecast_month(data = data_sc_4,
                                           system = system[sys],
                                           reference = reference[ref],
                                           var = var,
                                           start.year = start.year,
                                           end.year = end.year,
                                           start.months = start.months,
                                           forecast.months = forecast.months,
                                           region.names = region.names,
                                           metrics = metrics,
                                           scorecard_type = 4,
                                           fileout =  fileout)
        } ## close if
        
      } ## close loop on ref
    } ## close loop on sys
    
    print("All scorecard plots created")
        
} ## close function      
    
    

 