#' Scorecards function create simple scorecards by forecast month (types 2 & 4)
#' 
#'@description function to create scorecards for single system and reference 
#'  combination, showing data by forecast month and region.
#'  
#'@param data is an array of spatially aggregated metrics data, containing the
#'   following dimensions; system, reference, region, sdate, ftime, metric.
#'@param system a vector of character strings defining the names of the 
#'  system names following the archive.yml format from verification suite 
#'@param reference a vector of character strings defining the names of 
#'  the references following the archive.yml format from verification suite 
#'  Pending to be test with more than one.
#'@param var a character string following the format from 
#'  variable-dictionary.yml from verification suite 
#'@param start.year a numeric indicating the start year of the reference period
#'@param end.year a numeric indicating the end year of the reference period
#'@param start.date a vector of character strings indicating the start months
#'@param forecast.month a vector of numeric indicating the forecast months
#'@param region.names a vector of character strings containing names of the regions
#'@param metrics a vector of character strings containing names of the metrics, 
#'  must be in same order as in data.
#'@param title a character string of the desired output scorecard title
#'  
#'@example 
#'scorecard_plot <- SC_plot_simple(data = aggregated_metrics,
#'                                 system = c('ecmwfs5', 'dwds21')  ,
#'                                 reference = 'era5',
#'                                 var = 'tas',
#'                                 start.year = 1993,
#'                                 end.year = 2016,
#'                                 start.months = 1:12
#'                                 forecast.months = 1:6,
#'                                 region.names = c('Global', 'Europe'),
#'                                 metrics = c('Mean bias', 'Correlation', 'RPSS', 'CRPSS', 'Spread-to-Error Ratio'),
#'                                 column.header = 'Start date' ## or 'Target month'
#'                                 title = 'scorecard test')


## Make function independent of previous functions
## For use with verification suit and previous functions a wrapper will be developed

SC_plot_simple_by_forecast_month <- function(data,
                                             system,
                                             reference,
                                             var,
                                             start.year,
                                             end.year,
                                             start.months,
                                             forecast.months,
                                             region.names,
                                             metrics,
                                             scorecard_type,
                                             fileout){
  
  
  ## For testing:
  # data <- round(aggregated_metrics, 2)
  # system <- c('ecmwfs5', 'dwds21')  
  # reference <-'era5'
  # var <- 'tas'
  # start.year <- 1993
  # end.year <- 2016
  # start.months <- sprintf("%02d", 1:12)
  # forecast.months <- 1:6
  # region.names <- c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
  # metrics <- c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr')
  # fileout <- '/esarchive/scratch/nmilders/scorecards_data/saved_results/testing'
  
  ## Load configuration files
  archive <- read_yaml("./conf/archive.yml")$archive
  var_dict <- read_yaml("./conf/variable-dictionary.yml")$vars
  
  ## Round data
  data <- round(data,2)
  
  ## get dimensions of inputs
  n.start.months <- as.numeric(dim(data)["sdate"]) ## TO DO: create sdate_dim
  n.ftime.months <- as.numeric(dim(data)["time"]) ## TO DO: create ftime_dim
  n.metrics <- length(metrics) 
  n.regions <- length(region.names) 
  
  ## Define table size
  n.rows <- n.regions * n.ftime.months 
  n.columns <- 2 + (n.metrics * n.start.months)

  ## Get long names for showing in table
  var.name <- var_dict[[var]]$long_name
  var.units <- var_dict[[var]]$units
  system.name <- archive$System[[which(archive$System %like% system)]]$system_name
  reference.name <- archive$Reference[[reference]]$reference_name
  
  metric.names.list <- list(mean_bias = paste0("Mean bias (", var.units,")"),
                            enscorr = "Correlation",
                            rpss = "RPSS",
                            crpss = "CRPSS",
                            enssprerr = "Spread-to-error ratio")
  for(met in metrics){
    if(met == metrics[1]){
      metric.names <- metric.names.list[[met]] 
    } else {
      metric.names <- c(metric.names, metric.names.list[[met]])
    }
  }
  
  ## Region layout for plotting 
  forecast.month.figure <- rep("", n.regions * n.ftime.months)
  for(ft in 1:n.ftime.months) forecast.month.figure[(ft-1) * n.regions + 1] = ft
  #region.names.figure <- rep(region.names, n.ftime.months)
  
  
  ## Define table columns
   column.names <- c("Forecast month", "Region", rep(c(month.abb[as.numeric(start.months)]), n.metrics))

  if(type == 2){
    column.header <- "Start date"
  }
  if(type == 4){
    column.header <- "Target month"
  }
   
   options(stringsAsFactors = FALSE) 
   table.title <- data.frame(c1=paste0(var.name, " of ", system.name), c2 = n.columns)
   header.names <- as.data.frame(data.frame(c1=c("", metric.names), c2=c(2, rep(n.start.months, n.metrics)))) ## TO DO: need to include metric units
   header.names2 <- as.data.frame(data.frame(c1=c("", paste0(rep(column.header, n.metrics))), c2=c(2, rep(n.start.months, n.metrics))))
   ref.label <- data.frame(c1=paste0("(Ref: ", reference.name, " ", start.year, "-", end.year, ")"), c2 = n.columns)
   
   ## Define data inside the scorecards table
   ## NEED TO FIX: allow different or of metrics than in data
   for(ft in 1:n.ftime.months){
     table_metric <- data.frame(table_column_2 = as.character(regions.names))
     for(met in 1:n.metrics){
       table_metric <- data.frame(table_metric, data[met,,,ft])   
     }
     if(ft == 1){
       table_ft <- table_metric
     } else {
       table_ft <- rbind(table_ft, table_metric)
     }
   } ## close loop on ft

   ## All data for plotting in table
   table <- data.frame(table_column_1 = forecast.month.figure, table_ft)
   
   ## Find position of mean bias metric to calculate breaks
   if ('mean_bias' %in% metrics) {
     pos_bias <- which(metrics == 'mean_bias')
     breaks_bias <- SC_breaks(Subset(data, along = 'metric', 
                                     indices = pos_bias))
   }
   
   ## Define breaks for each metric based of metric position:
   metric.breaks <- list('mean_bias' = breaks_bias,
                         'enscorr' = c(-1, -0.8, -0.6, -0.4, -0.2, 0, 0.2, 0.4, 0.6, 0.8, 1),
                         'rpss' = c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5),
                         'crpss' = c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5),
                         'enssprerr' = c(0, 0.6, 0.7, 0.8, 0.9, 1, 1.2, 1.4, 1.6, 1.8, 2))
   
   
   metric.col <- array(colorunder, c(n.ftime.months * n.regions, n.columns-2)) 
   metric.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.columns-2)) ## black is the default text color
   # metric.text.bold <- array(TRUE, c(n.ftime.months * n.regions, n.columns-2)) ## Setting all values to bold
   
   ## temporary table in array format
   table_temp <- array(unlist(table[3:n.columns]), dim=c(n.rows,n.columns-2)) ##as.matrix(table[3:n.columns])
   
   ## Set table background colors
   palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
   colorunder <- "#04040E"
   colorsup <- "#730C04"
   
   ## Define cell and text colors to show in table
   for(met in 1:n.metrics){
     metric.int <- metric.breaks[[metrics[met]]]
     for(rr in 1:n.rows){
       for(stm in 1:n.start.months){
         for(pp in 1:(length(metric.int)-1)){
           if(is.nan(table_temp[rr,((met - 1) * n.start.months + stm)])){
             metric.col[rr,((met - 1) * n.start.months + stm)] <- "gray"
           } else {
             if(table_temp[rr,((met - 1) * n.start.months + stm)] >= metric.int[pp] && table_temp[rr,((met - 1) * n.start.months + stm)] <= metric.int[pp+1])
               metric.col[rr,((met - 1) * n.start.months + stm)] <- palette[pp]
             
             if(table_temp[rr,((met - 1) * n.start.months + stm)] < metric.int[1])
               metric.col[rr,((met - 1) * n.start.months + stm)] <- colorunder
             if(table_temp[rr,((met - 1) * n.start.months + stm)] >= metric.int[length(metric.int)])
               metric.col[rr,((met - 1) * n.start.months + stm)] <- colorsup
           }
           
           ## color text in white and bold if background is white or dark blue or dark red:
           if(is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) ||
              (!is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) && pp == 1 && table_temp[rr,((met - 1) * n.start.months + stm)] < metric.int[2]) ||
              (!is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) && pp == 2 && table_temp[rr,((met - 1) * n.start.months + stm)] < metric.int[3]) ||
              (!is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) && pp == (length(metric.int) - 1) && table_temp[rr,((met - 1) * n.start.months + stm)] >= metric.int[length(metric.int) - 1]) ||
              (!is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) && pp == (length(metric.int) - 2) && table_temp[rr,((met - 1) * n.start.months + stm)] >= metric.int[length(metric.int) - 2]))
           {
             metric.text.col[rr,((met - 1) * n.start.months + stm)] <- "white"
             #metric.text.bold[rr,((met - 1) * n.start.months + stm)] <- TRUE
           }
         } ## close loop on pp
       } ## close loop on stm
     } ## close loop on rr
   } ## close loop on met
   
   
   metric.col.df <- data.frame(metric.col)
   metric.text.col.df <- data.frame(metric.text.col)
   #metric.text.bold.df <- data.frame(metric.text.bold)
   
   ## Create HTML table
   table.html.part <- list()
   table.html.part[[1]] <- kbl(table, escape = F, col.names = column.names, align = rep("c", n.columns)) %>%
     kable_paper("hover", full_width = F) %>%
     add_header_above(header = header.names2, font = 14)%>%
     add_header_above(header = ref.label, align = "left", font_size = 14) %>% 
     add_header_above(header = header.names, font_size = 20) %>%
     add_header_above(header = table.title, font_size = 20, align = "left")
   
   for(i in 1:n.metrics){ 
     for(stm in 1:n.start.months){ 
       my.background <- metric.col.df[(i - 1) * n.start.months + stm]
       my.text.color <- metric.text.col.df[(i - 1) * n.start.months + stm ]
       #my.bold <- metric.text.bold.df[(i - 1) * n.start.months + stm]
       
       table.html.part[[(i - 1) * n.start.months + stm + 1]] <-
         column_spec(table.html.part[[(i - 1) * n.start.months + stm]], 2 + n.start.months * (i - 1) + stm, 
                     background = my.background[1:n.rows,], 
                     color = my.text.color[1:n.rows,], 
                     bold = T) ## strsplit(toString(bold), ', ')[[1]]
     } ## close loop on stm
   } ## close loop on i
   
   n.last.list <- n.metrics * n.start.months + 1
   
   table.html <- column_spec(table.html.part[[n.last.list]], 1, bold = TRUE, width_min = "1cm", width_max = "1cm") %>%
     column_spec(1, border_left = "4px solid black") %>% 
     column_spec(2 , bold = TRUE, width_min = "4cm", width_max = "4cm") %>%
     # column_spec(3:n.columns, width_max = "0.9cm", width_min = "0.9cm") %>% ## columns to small
     column_spec(c(1, 2 , 2  + n.start.months, 2  + n.start.months*2, 2  + n.start.months*3,  2 + n.start.months*4), border_right = "1px solid black") %>%
     ## TO DO: make flexible
     column_spec(n.columns, border_right = "4px solid black") %>% 
     row_spec(seq(from = 0, to = n.ftime.months * n.regions, by = n.regions), extra_css = "border-bottom: 2px solid black", hline_after = TRUE)
   
   
   ## Create color bar legends
   if('mean_bias' %in% metrics){
     png(filename= paste0(fileout, "/scorecard_legend_mean_bias.png"), width=455, height=40)
     ColorBar(brks = metric.breaks[['mean_bias']], cols = palette, vertical = FALSE, triangle_ends_scale = 1, col_inf = colorunder, col_sup = colorsup) 
     dev.off()
   }
   if('enscorr' %in% metrics){
     png(filename= paste0(fileout, "/scorecard_legend_enscorr.png"), width=455, height=40)
     ColorBar(brks = metric.breaks[['enscorr']], cols = palette, vertical = FALSE, triangle_ends_scale = 0, col_inf = colorunder)
     dev.off()
   }
   if('rpss' %in% metrics){
     png(filename= paste0(fileout, "/scorecard_legend_rpss.png"), width=455, height=40)
     ColorBar(brks = metric.breaks[['rpss']], cols = palette, vertical = FALSE, var_limits=c(-10000000, 1), col_inf = colorunder, col_sup = colorsup) 
     dev.off()
   }
   if('crpss' %in% metrics){
     png(filename= paste0(fileout, "/scorecard_legend_crpss.png"), width=455, height=40)
     ColorBar(brks = metric.breaks[['crpss']], cols = palette, vertical = FALSE, var_limits=c(-10000000, 1), col_inf = colorunder, col_sup = colorsup) 
     dev.off()
   }
   if('enssprerr' %in% metrics){
     png(filename= paste0(fileout, "/scorecard_legend_enssprerr.png"), width=455, height=40)
     ColorBar(brks = metric.breaks[['enssprerr']], cols = palette, vertical = FALSE, triangle_ends_scale = 1,  col_sup = colorsup)
     dev.off()
   }
   
   currentworkingdir <- getwd()
   setwd(fileout)
   
   ## Paste all legends together
   for(m in metrics){
     if(m == metrics[1]){
       ## Add white space to the left of the first color bar legend
       system(paste0("convert ./scorecard_legend_", m, ".png -background white -splice 225x0 ./legend_scorecard.png"))
     } else {
       system(paste0("montage ./legend_scorecard.png ./scorecard_legend_",m,".png -tile 2x1 -geometry -4+10 ./legend_scorecard.png"))
     }
   }
   unlink('scorecard_legend*')
   
   ## Save html table as png
   my.file <- paste0(fileout, '/scorecard.png')
   try(save_kable(table.html, file = my.file), silent = TRUE)
   
   ## Remove temporal directories (not automatically removed due to error with save_kable)
   unlink(paste0(fileout, './saved_results/lib*'), recursive = TRUE) # clean temporal directories (not automatically removed because of error with save_kable)
   
   ## Add white space to the top of the scorecards, to be cut when adding legend:
   setwd(fileout)
   system(paste0("convert ./scorecard.png -background white -splice 0x20 ./scorecard.png"))
   
   ## Add the legends below the scorecard table
   system(paste0("montage ./scorecard.png ./legend_scorecard.png -tile 1x2 -geometry +0-10 ./scorecard.png"))
   
   
   ## Save scorecard png in desired directory
   dirout <- initialfunction_save_scorecard(system, reference, var, start.year, end.year, scorecard_type)
   system(paste0("mv ", './scorecard.png', paste0(" ", dirout))) 
   print(paste0("Scorecard type ", scorecard_type, " figure saved (sys: ", system, " & ref: ", reference, ")" ))
   
   setwd(currentworkingdir)
 
}## close function
